package org.springframework.data.relational.core.sql.render;

import org.springframework.data.relational.core.sql.FunctionCondition;
import org.springframework.data.relational.core.sql.SimpleFunction;
import org.springframework.data.relational.core.sql.Visitable;
import org.springframework.lang.Nullable;

public class FunctionVisitor extends FilteredSubtreeVisitor {

    private final RenderContext context;
    private final RenderTarget target;
    private @Nullable PartRenderer current;
    private final StringBuilder part = new StringBuilder();

    public FunctionVisitor(FunctionCondition condition, RenderContext context, RenderTarget target) {
        super(it -> it == condition);
        this.context = context;
        this.target = target;
    }

    @Override
    Delegation enterNested(Visitable segment) {
        if (segment instanceof SimpleFunction) {
            SimpleFunctionVisitor visitor = new SimpleFunctionVisitor(context);
            current = visitor;
            return Delegation.delegateTo(visitor);
        }
        throw new IllegalStateException("Cannot provide visitor for " + segment);
    }

    @Override
    Delegation leaveNested(Visitable segment) {
        if (current != null) {
            part.append(current.getRenderedPart());
            current = null;
        }
        return super.leaveNested(segment);
    }

    @Override
    Delegation leaveMatched(Visitable segment) {
        target.onRendered(part);
        return super.leaveMatched(segment);
    }
}
